#!/bin/bash
# setting up irq affinity according to /proc/interrupts
# adapted from the one from intel driver
# version 0.1:  mjqiu <mjqiu@tencent.com> 
#      Initial version, a best effort to set cpu affinity for both intel and bnx2 drivers
# version 0.2:  lynnsong <lynnsong@tencent.com>
#      Added chelsio netcard support
# version 0.3:  lynnsong <lynnsong@tencent.com>
#      Don't kill irqbalance if there is no mutiqueue netcard
# version 0.4: Chapman Ou <chapmanou@tencent.com>
#      Port to tlinux 2.0	  
# version 0.5: yama <yama@tencent.com>
#      Set up single queue NIC only
# version 0.6: Songqiao Tao <joeytao@tencent.com>
#      fix the smp_affinity value bug when cpus are more than 32
# version 0.7: Songqiao Tao <joeytao@tencent.com>
#	Added Intel 40GbE netcard support
# version 0.8: Songqiao Tao <joeytao@tencent.com>
#	Ensure the irqbalance service is stopped
# version 0.9: Lin Yi <nilusyi@tencent.com>
#   Added Mellanox netcard support
version=0.9

set_affinity()
{
    VEC=$queue
    if [ $VEC -ge 32 ]; then
        MASK_FILL=""
        MASK_ZERO="00000000"
        let "IDX = $VEC / 32"
        for ((i=1; i<=$IDX;i++))
        do
            MASK_FILL="${MASK_FILL},${MASK_ZERO}"
        done

        let "VEC -= 32 * $IDX"
        MASK_TMP=$((1<<$VEC))
        MASK=`printf "%X%s" $MASK_TMP $MASK_FILL`
    else
        MASK_TMP=$((1<<$((queue))))
        MASK=`printf "%X" $MASK_TMP`
    fi
    printf "%s mask=%s for /proc/irq/%d/smp_affinity\n" $DEV $MASK $IRQ
    printf "%s" $MASK > /proc/irq/$IRQ/smp_affinity
}

set_intel_broadcom()
{
	#  Intel##########
	#  Assuming a device with two RX and TX queues.
	#  This script will assign: 
	#
	#	eth0-rx-0  CPU0
	#	eth0-rx-1  CPU1
	#	eth0-tx-0  CPU0
	#	eth0-tx-1  CPU1
	# Intel###########
	
	# broadcom########
	# bnx2:
	# eth0-0 eth0-1 eth0-2 eth0-3 eth0-4
	# eth0-0 eth0-1 eth0-2 eth0-3
	# bnx2x:
	# eth0-rx-0 eth0-rx-1 ...
	# eth0-tx-0 eth0-tx-1 ...
	# broadcom########

	local MAX_CPU=$(( $(grep processor /proc/cpuinfo | wc -l) ))
	cd /proc/irq
	ls -d */*eth* | sed 's/i40e//g' | sed 's/[^0-9][^0-9]*/ /g' > /tmp/irqaffinity
	while read irq eth queue
	do
		if [ -n "${queue}" ]; then
			queue=$(($[queue%${MAX_CPU}]))
			IRQ=$irq
			DEV="eth"$eth
			set_affinity
		fi
	done < /tmp/irqaffinity

}
set_chelsio()
{
	local MAX_CPU=$(( $(grep processor /proc/cpuinfo | wc -l)-1 ))
	queue=$((0))
	for DEV in $(ifconfig -a | grep 00:07:43 | awk '{print $1}'); do
		for IRQ in $(egrep "rdma|ofld|${DEV}" /proc/interrupts | awk '{printf "%d\n",$1}'); do
			set_affinity
			queue=$(($(($queue+1))%${MAX_CPU}))
		done
		ethtool -K $DEV gro off
	done	
}

if [ "$1" = "--version" -o "$1" = "-V" ]; then
	echo "version: $version"
	exit 0
elif [ -n "$1" ]; then
	echo "Description:"
	echo "    This script attempts to bind each queue of a multi-queue NIC"
	echo "    to the same numbered core, ie tx0|rx0 --> cpu0, tx1|rx1 --> cpu1"
	echo "usage:"
	echo "    $0 "
	exit 0
fi

# Set up the desired devices.
# check for irqbalance running
IRQBALANCE_ON=`ps ax | grep -v grep | grep -q irqbalance; echo $?`
if [ "$IRQBALANCE_ON" == "0" ] ; then
	echo "WARNING: irqbalance is running and will"
	echo "          likely override this script's affinitization."
        echo "          So I stopped the irqbalance service by"
        echo "          'service irqbalance stop'."
	service irqbalance stop
fi


result=$(ifconfig -a | grep "00:07:43")

if [ -n "${result}" ]; then
	set_chelsio
else
	set_intel_broadcom
fi

