#!/bin/bash
# setting up megasas affinity according to /proc/interrupts
# version 0.1: Songqiao Tao <joeytao@tencent.com>
#	Initial version, adapted from shoufenglai.
# 	fix the smp_affinity value bug when cpus are more than 32

version=0.1

set_affinity()
{
    VEC=$queue
    if [ $VEC -ge 32 ]; then
        MASK_FILL=""
        MASK_ZERO="00000000"
        let "IDX = $VEC / 32"
        for ((i=1; i<=$IDX;i++))
        do
            MASK_FILL="${MASK_FILL},${MASK_ZERO}"
        done

        let "VEC -= 32 * $IDX"
        MASK_TMP=$((1<<$VEC))
        MASK=`printf "%X%s" $MASK_TMP $MASK_FILL`
    else
        MASK_TMP=$((1<<$((queue))))
        MASK=`printf "%X" $MASK_TMP`
    fi
    printf "%s mask=%s for /proc/irq/%d/smp_affinity\n" $DEV $MASK $IRQ
    printf "%s" $MASK > /proc/irq/$IRQ/smp_affinity
}


set_megasas()
{
	local MAX_CPU=$(( $(grep processor /proc/cpuinfo | wc -l) ))
	ls -d /proc/irq/*/megasas* | sed 's/[^0-9][^0-9]*/ /g' >/tmp/irqaffinity
	while read irq dev queue
	do
		if [ -n "${queue}" ]; then
			queue=$(($[queue%${MAX_CPU}]))
			IRQ=$irq
			DEV="megasas""$dev"
			set_affinity
		fi
	done < /tmp/irqaffinity
}
if [ "$1" = "--version" -o "$1" = "-V" ]; then
        echo "version: $version"
        exit 0
elif [ -n "$1" ]; then
        echo "Description:"
        echo "    This script attempts to bind each queue of a multi-queue megasas to one cpu"
        echo "usage:"
        echo "    $0 "
        exit 0
fi

megasas_irq_num=`ls -d /proc/irq/*/megasas* | wc -l`
if [ $megasas_irq_num -gt 1 ]; then
	if [ `lspci | grep "Broadcom / LSI MegaRAID SAS-3 3108" | wc -l` -eq 1 ]; then
		echo "The Broadcom / LSI MegaRAID SAS-3 3108 [Invader] driver does not support the re-assignment of interrupts."
		exit 0
	fi
        set_megasas
fi
