#!/usr/libexec/platform-python
# -*- coding: utf-8 -*-

import os

def call(command):
    return os.popen(command).read()

def getNUMA():
    numaList = call("ls -d /sys/devices/system/node/node*").splitlines()
    return numaList

def getNUMADistanceMap():
    numaPaths = getNUMA()
    numaDistancePaths = [numaPath + "/distance" for numaPath in numaPaths]
    numaCpumaps = [call("cat %s/cpumap" % e).strip() for e in numaPaths]
    distanceMap = []
    minor = []
    for e in numaDistancePaths:
        distances = call("cat %s" % e).split()
        distanceMap.append(distances)
        for d in distances: 
            if d not in minor: minor.append(d)
        
    minor.sort()
    if len(minor) > 2:
        minor = minor[:2]
    else: minor = minor[:1]

    return numaCpumaps, minor, distanceMap

def getTargetNodeAffinity(numaCpumaps, localInfo):
    current = []
    for i in xrange(len(localInfo)):
        if localInfo[i] == 1:
            if current == []:
                current = list(numaCpumaps[i])
            else:
                for j in xrange(len(numaCpumaps[i])):
                    current[j] = current[j] if current[j] == 'f' else numaCpumaps[i][j]

    return ''.join(current)
            

def getTargetAffinity():
    numaCpumaps, minor, distanceMap = getNUMADistanceMap()
    for i in xrange(len(distanceMap)):
        for j in xrange(len(distanceMap)):
            if distanceMap[i][j] in minor: distanceMap[i][j] = 1
            else: distanceMap[i][j] = 0
    
    affinityMap = []
    for distance in distanceMap:
        affinityMap.append(getTargetNodeAffinity(numaCpumaps, distance))
    
    return affinityMap

def setRps():
    affinityMap = getTargetAffinity()

    eths = call("ls -d /sys/class/net/eth*").splitlines()
    for eth in eths:
        numaNode = int(call("cat %s/device/numa_node" % eth).strip())
        queues = call("ls -d %s/queues/rx-*" % eth).splitlines()
        for queue in queues:
            cmd = "echo %s > %s/rps_cpus" % (affinityMap[numaNode], queue)
            print(cmd)
            call(cmd)

def isVM():
    return "0" == call("lscpu| grep -i kvm &>/dev/null;echo $?").strip()

if __name__ == "__main__":
    if isVM():
        print("use default opencloudos_set_rps_cpus.py for VM")
        with open('/usr/sbin/opencloudos_set_rps_cpus.py','r') as f:
            exec(f.read())
    else:
        setRps()
